using System;
using System.Data;
using NHibernate.SqlTypes;

namespace NHibernate.Type
{
	/// <summary>
	/// Maps a <see cref="System.DateTime" /> property to a <see cref="DbType.DateTime"/> column that
	/// stores date &amp; time down to the accuracy of the database.
	/// </summary>
	/// <remarks>
	/// If you are looking for the most accurate date and time storage accross databases use the
	/// <see cref="TicksType"/>. If you are looking for the Hibernate <c>DateTime</c> equivalent,
	/// use the <see cref="DateTimeNoMsType" />.
	/// </remarks>
	[Serializable]
	public class DateTimeType : AbstractDateTimeType
	{
		/// <summary>
		/// Default constructor.
		/// </summary>
		public DateTimeType()
		{
		}

		/// <summary>
		/// Constructor for specifying a datetime with a scale. Use <see cref="SqlTypeFactory.GetDateTime"/>.
		/// </summary>
		/// <param name="sqlType">The sql type to use for the type.</param>
		public DateTimeType(DateTimeSqlType sqlType) : base(sqlType)
		{
		}

		// TODO: NH5 review this change (added by ThoHot)
		//   NH5 changed the default data type mapping for "DateTime" from the previous DateTimeNoMsType
		//   to DateTimeType. This changes has the side-effect that the DateTime.Kind value returned by
		//   the database is preserved and no longer replaced by "Unspecified".
		//   To keep compatibility with previous behavior this change sets the DateTime.Kind value again
		//   to unspecified.
		//   ATTENTION: Therefore this will prevent that the database preservers the kind of date times.
		/// <inheritdoc />
		protected override DateTime AdjustDateTime(DateTime dateValue) =>
			base.AdjustDateTime(DateTime.SpecifyKind(dateValue, Kind));

		/// <inheritdoc />
		public override string Name => "DateTime";
	}
}
